-- Import
local addonName, addonData = ...;

-- Globals --
DLU = {
	prefix = "[DLU]",
	addonName = addonName,
	addonNameColored = addonName,
	addonVersion = "0",
	aprilsFool = false
}
DLU.addonColor = GetAddOnMetadata(DLU.addonName, "X-AddonColor");
DLU.addonNameColored = format("|c%s%s|r", DLU.addonColor, DLU.addonName);
DLU.addonVersion = GetAddOnMetadata(DLU.addonName, "Version");

-- Frames
CreateFrame("Frame", "DLUCoreFrame");

-- Settings
function DLU.getDefaultOptions()
	return {
		showDLUHQOnLogon = {
			text = "Show DarkruneDK HQ on start up",
			value = false
		},
		showLoadMessage = {
			text = "Show message when addon is loaded",
			value = true
		},
		numberFormat = "Blizzard"
	};
end

function DLU.getDefaultCharacterSettings()
	return {
		questingMode = {
			text = "Questing mode",
			value = false
		},
		currencies = {
			[1] = 9999,
			[2] = 9999,
			[3] = 9999,
			[4] = 9999,
			[5] = 9999
		},
		autoRepair = {
			text = "Enable auto-repair",
			value = false
		}
	};
end

function DLU.getSettings()
	DLUSettings = DLUSettings or DLU.getDefaultOptions();
	DLUCharacterSettings = DLUCharacterSettings or DLU.getDefaultCharacterSettings();
end

function DLU.resetAddonOptions()
	DLUSettings = DLU.getDefaultOptions();
end

function DLU.resetCharacterOptions()
	DLUCharacterSettings = DLU.getDefaultCharacterSettings();
end

-- DLU functions
function DLU.tableContains(tableToCheck, element)
	if (tableToCheck ~= nil) then
		for i = 0, #tableToCheck do
			if (element == tableToCheck[i]) then
				return tableToCheck[i];
			end
		end
	end

	return false;
end

function DLU.tableLength(tableToCheck)
	local count = 0;
	for _ in pairs(tableToCheck) do count = count + 1 end
	return count;
end

function DLU.tableMerge(t1, t2)
	for k,v in ipairs(t2) do
		table.insert(t1, v);
	end
end

function DLU.colorizeString(text, colorType)
	if (colorType == "red") then
		return "|cffCC0000" .. text .. "|r";
	elseif (colorType == "green") then
		return "|cff00CC00" .. text .. "|r";
	elseif (colorType == "blue") then
		return "|cff0000CC" .. text .. "|r";
	elseif (colorType == "orange") then
		return "|cffffa500" .. text .. "|r";
	elseif (colorType == "black") then
		return "|cff000000" .. text .. "|r";
	elseif (colorType == "purple") then
		return "|cff00ffff" .. text .. "|r";
	elseif (colorType == "darkgray") then
		return "|cff444444" .. text .. "|r";
	elseif (colorType == "expansion") then
		return format("|c%s%s|r", DLU.addonColor, text);
	else
		return text;
	end
end

function DLU.createMessage(message, messageType)
	local prefixText = DLU.colorizeString(DLU.prefix, "expansion");
	local text = format("%s %s", prefixText, message);

	if (messageType == "error") then
		text = format("%s %s", prefixText, DLU.colorizeString(message, "red"));
	end

	print(text);
end

function DLU.numberFormat(number)
	if (number == nil or number <= 0) then
		return 0;
	elseif (number > 0 and number < 1000) then
		return number;
	elseif (number > 1000 and  number < 1000000) then
		return format("%.1f%s", number * 0.001, "K");
	elseif (number >= 1000000 and number < 1000000000) then
        return format("%.1f%s", number * 0.000001, "M");
    elseif (number >= 1000000000) then
        return format("%.1f%s", number * 0.000000001, "B");
	elseif (number >= 1000000000000) then
		return format("%.1f%s", number * 0.000000000001, "T");
	end

	return number;
end

function DLU.RGBPercToHex(r, g, b)
	r = r <= 1 and r >= 0 and r or 0
	g = g <= 1 and g >= 0 and g or 0
	b = b <= 1 and b >= 0 and b or 0
	return string.format("%02x%02x%02x", r*255, g*255, b*255)
end

function DLU.itemColorString(num, text)
	if (text ~= nil) then
		local color = ITEM_QUALITY_COLORS[num];
		local result = color.hex .. text .. "|r";
		return result;
	end
	
	return;
end

function DLU.iconToText(iconPath)
	local result = format("|T%s:0|t", iconPath);
	return result;
end

function DLU.iconToTextSized(iconPath, height, width)
	local result = format("|T%s:%i:%i:0:0:64:64:4:60:4:60|t", iconPath, height, width);
	return result;
end

function DLU.createNotification(message, notificationType)
	local sounds = {
		["Invasion"] = 14772,
		["Default"] = 4574
	};
	
	local sound = sounds["Default"];
	
	if (sounds[notificationType]) then
		sound = sounds[notificationType];
	end
	
	PlaySound(sound);
	UIFrameFlash(DLUNotificationFrame, 3, 3, 16, false, 10, 0);
	DLUNotification:SetText(message);
end

function DLU.boolToText(boolValue)
	if (boolValue) then
		return DLU.colorizeString("Yes", "green");
	end

	return DLU.colorizeString("No", "red");
end

function DLU.tableStatus(tableToCheck)
	local completionAmount = 0;
	for t, q in pairs(tableToCheck) do
		local completed = IsQuestFlaggedCompleted(q);
		if (completed) then
			completionAmount = completionAmount + 1;
		end

		local result = DLU.boolToText(completed);
		print(format("%s: %s", t, result));
	end

	print(format("You have completed %i/%i", completionAmount, DLU.tableLength(tableToCheck)));
end

function DLU.tableStatusOrdered(tableToCheck)
	local completionAmount = 0;
	for t, q in pairs(tableToCheck) do
		local completed = IsQuestFlaggedCompleted(q.value);
		if (completed) then
			completionAmount = completionAmount + 1;
		end

		local result = DLU.boolToText(completed);
		print(format("%s: %s", q.name, result));
	end

	print(format("You have completed %i/%i", completionAmount, DLU.tableLength(tableToCheck)));
end

function DLU.ArtifactXpLeftValue()
	local _, _, _, _, spendPower, currentTraits, _, _, _, _, _, _, artifactTier = C_ArtifactUI.GetEquippedArtifactInfo();
	
	if (spendPower ~= nil) then
		local traitsWaitingForSpending, currentPower, powerForNextTrait = ArtifactBarGetNumArtifactTraitsPurchasableFromXP(currentTraits, spendPower, artifactTier);
		local apNeeded = powerForNextTrait - currentPower;
		local apPercentGained = math.floor((currentPower / powerForNextTrait) * 100);
		local apPercentNeeded = 100 - apPercentGained;
		
		local apFormat = BreakUpLargeNumbers(apNeeded);
		if (DLUSettings.numberFormat == "DarkruneDK") then
			apFormat = DLU.numberFormat(apNeeded);
		end
		
		return format("%s", apFormat), apPercentNeeded;
	end
	
	return nil;
end

function DLU.ArtifactTraits()
	local _, _, _, _, spendPower, currentTraits, _, _, _, _, _, _, artifactTier = C_ArtifactUI.GetEquippedArtifactInfo();
	
	if (spendPower ~= nil) then
		local traitsWaitingForSpending, currentPower, powerForNextTrait = MainMenuBar_GetNumArtifactTraitsPurchasableFromXP(currentTraits, spendPower, artifactTier);
		local currentRank = currentTraits + traitsWaitingForSpending;
		
		return currentRank, traitsWaitingForSpending, currentRank + 1;
	end
	
	return nil;
end

function DLU.ArtifactInfo()
	local _,_,name,_, spendPower, currentRank,_,_,_,_,_,_, artifactTier = C_ArtifactUI.GetEquippedArtifactInfo();
	if (name ~= nil and spendPower ~= nil) then
		return name, spendPower, currentRank;
	end
	
	return nil;
end

function DLU.XpLeft()
	local xpToNextLevel = (UnitXPMax("player") - UnitXP("player"));
	local xpRested = GetXPExhaustion();
	local xpRestedPercentage;
	local xpRestedFormat;
	
	if (xpRested ~= nil) then
		xpRestedPercentage = floor(xpRested / UnitXPMax("player")) * 100;
	end

	local xpPercentageGained = floor((UnitXP("player") / UnitXPMax("player")) * 100);
	local xpPercentageNeeded = 100 - xpPercentageGained;

	local xpDisabled = IsXPUserDisabled();
	local maxLevel = GetMaxPlayerLevel();
	
	local xpFormat = BreakUpLargeNumbers(xpToNextLevel);
	if (DLUSettings.numberFormat == "DarkruneDK") then
		xpFormat = DLU.numberFormat(xpToNextLevel);
	end
	
	if (xpRested) then
		xpRestedFormat = BreakUpLargeNumbers(xpRested);
		if (DLUSettings.numberFormat == "DarkruneDK") then
			xpRestedFormat = DLU.numberFormat(xpRested);
		end
	end
	
	return xpFormat, xpRestedFormat, xpPercentageNeeded, xpRestedPercentage, xpDisabled, maxLevel;
end

function DLU.XpLabelInfo()
	local nextLevel = UnitLevel("player") + 1;
	local maxLevel = GetMaxPlayerLevel();
	
	return nextLevel, maxLevel;
end

function DLU.honorLeftValues()
	local honorLeft = UnitHonorMax("player") - UnitHonor("player");
	local honorLevel = UnitHonorLevel("player");
	
	local honorKills = GetPVPLifetimeStats();
	
	local blizzFormatHonorLeft = BreakUpLargeNumbers(honorLeft);
	
	local honorKillsFormatted = BreakUpLargeNumbers(honorKills);
	local xpFormat = blizzFormatHonorLeft;
	if (DLUSettings.numberFormat == "DarkruneDK") then
		xpFormat = DLU.numberFormat(honorLeft);
		honorKillsFormatted = DLU.numberFormat(honorKills);
	end
	
	return xpFormat, honorLevel, blizzFormatHonorLeft, honorKillsFormatted;
end

function DLU.getMovementSpeedInfo()
	local _, groundSpeed, flightSpeed, swimSpeed = GetUnitSpeed("player");

	local playerGroundSpeed = floor((groundSpeed / BASE_MOVEMENT_SPEED) * 100);
	local playerFlySpeed = floor((flightSpeed / BASE_MOVEMENT_SPEED) * 100);
	local playerSwimSpeed = floor((swimSpeed / BASE_MOVEMENT_SPEED) * 100);

	local data = {
		ground = playerGroundSpeed,
		flying = playerFlySpeed,
		swimming = playerSwimSpeed
	};

	return data;
end

function DLU.getAzeriteArtifactInfo()
	local azeriteEquipped = C_AzeriteItem.HasActiveAzeriteItem();
	
	local azeriteInfo = {
		isEquipped = azeriteEquipped
	};
	
	if (azeriteEquipped) then
		local azeriteItemLocation = C_AzeriteItem.FindActiveAzeriteItem();
		
		local currentXp, totalXp = C_AzeriteItem.GetAzeriteItemXPInfo(azeriteItemLocation);
		local level = C_AzeriteItem.GetPowerLevel(azeriteItemLocation);
		local xpNextLevel = totalXp - currentXp;
		
		local azeriteItem = Item:CreateFromItemLocation(azeriteItemLocation);
		
		azeriteInfo.name = azeriteItem:GetItemName();
		azeriteInfo.level = level;
		azeriteInfo.xp = currentXp;
		azeriteInfo.totalXp = totalXp;
		
		local xpPercentage = math.floor((currentXp / totalXp) * 100);
		azeriteInfo.xpPercentage = xpPercentage;
		
		if (DLUSettings.numberFormat == "DarkruneDK") then
			xpNextLevel = DLU.numberFormat(xpNextLevel);
		end
		
		azeriteInfo.xpNextLevel = xpNextLevel;
	end
	
	return azeriteInfo;
end

local function getInventoryItemLevels()
	local averageIlvl, equippedIlvl, pvpIlvl = GetAverageItemLevel();
	local multiplyer = 100;
	averageIlvl = floor(multiplyer * averageIlvl)/multiplyer;
	equippedIlvl = floor(multiplyer * equippedIlvl)/multiplyer;
	pvpIlvl = floor(multiplyer * pvpIlvl) / multiplyer;
	local iLvls = {total = averageIlvl, equipped = equippedIlvl, pvp = pvpIlvl};
	return iLvls;
end

function DLU.getItemLevel()
	local iLvls = getInventoryItemLevels();
	if (UnitInBattleground("player")) then
		return iLvls.pvp;
	else
		return iLvls.equipped;
	end
end

-- local variables
local helpList;

local function ArtifactXpLeft()
	
	local apNeeded, apPercentNeeded = DLU.ArtifactXpLeftValue();
	local currentRank, traitsWaitingForSpending, nextRank = DLU.ArtifactTraits();
	
	if (apNeeded ~= nil and currentRank ~= nil) then
		
		local specId, _, _, _, role = GetSpecializationInfo(GetSpecialization());
		local artifactLink = GetInventoryItemLink("player", 16);
		if (specId == 73 or specId == 66) then
			artifactLink = GetInventoryItemLink("player", 17);
		end
		
		local upgradeText = "upgrade";
		local text = "You need %s AP (%i%%) to get %s to rank: %i";
		if (traitsWaitingForSpending > 0) then
			text = text .. " (%i trait %s)";
			if (traitsWaitingForSpending > 1) then
				upgradeText = upgradeText .. "s";
			end
		end
		
		return format(text, apNeeded, apPercentNeeded, artifactLink, nextRank, traitsWaitingForSpending, upgradeText);
	end
	
	return false;
end

-- Character Stats
DLU.StatFrames = {
	["ItemLevelFrame"] = {
		frame = CharacterStatsPane.ItemLevelFrame,
		updateFunc = function(statFrame, unit)
			ilvl = DLU.getItemLevel();
			PaperDollFrame_SetLabelAndText(statFrame, STAT_AVERAGE_ITEM_LEVEL, ilvl, false, ilvl);
			statFrame.tooltip = HIGHLIGHT_FONT_COLOR_CODE .. format("Item Level: %s", ilvl) .. FONT_COLOR_CODE_CLOSE;
		end
	}
}

function DLU.checkStats()
	local addonList = {"DejaCharacterStats"}; -- List of addons, that already changes the char stats window
	local disableCharStats = false;
	
	for _, v in pairs(addonList) do
		local _, _, _, enabled = GetAddOnInfo(v);
		if (enabled) then
			disableCharStats = true;
			break;
		end
	end
	
	return disableCharStats;
end

local hideStats = DLU.checkStats();
if (not hideStats) then
	for k, v in pairs(DLU.StatFrames) do
		local index = 0;
		if (not v.frame) then
			local category = PAPERDOLL_STATCATEGORIES[1];
			local categoryFrame = category.categoryFrame;

			for _, v in pairs(CharacterStatsPane[categoryFrame]) do
				if (v) then
					index = index+1;
				end
			end

			v.frame = CreateFrame("Frame", nil, CharacterStatsPane[categoryFrame], "CharacterStatFrameTemplate");
			local height = v.frame:GetHeight();
			local xCord = -(index*height);
			v.frame:SetPoint("BOTTOMLEFT", 4, xCord);
			index = index+1;
		end
	end
end

function DLU.showStats(unit)
	if (not hideStats) then
		for _, v in pairs(DLU.StatFrames) do
			if (v.updateFunc) then
				if (v.frame) then
					v.updateFunc(v.frame, unit);
				end
			end
		end
	end
end

local function testIt()
	local spec = GetSpecialization();
	local talentTemplate = {};
	talentTemplate.name = name;
	talentTemplate.talents = {};
	
	for i = 1, MAX_TALENT_TIERS do
		for j = 1, NUM_TALENT_COLUMNS do
			local id, name, _, selected = GetTalentInfoBySpecialization(spec, i, j);
			print(selected);
			
			if (selected) then
				table.insert(talentTemplate.talents, id);
			end
		end
	end
	
	--[[
	for k,v in pairs(DLU.player) do
		print(k .. ":" .. v);
	end
	]]
end

local function createHelpList()
	helpList = {
		[1] = {
			["Command"] = SLASH_RELOADUI1,
			["Explanation"] = "Short version of the in-build /reload"
		},
		[2] = {
			["Command"] = SLASH_DLU1 .. " ms",
			["Explanation"] = "Tells the characters different types of movement speed"
		},
		[3] = {
			["Command"] = SLASH_DLU1 .. " il or " .. SLASH_DLU1 .. " ilvl",
			["Explanation"] = "Tells you the characters item level"
		},
		[4] = {
			["Command"] = SLASH_DLU1 .. " cm",
			["Explanation"] = "Checks if the character you are currently playing has unlocked it's class mount"
		},
		-- UI
		[5] = {
			["Command"] = SLASH_DLU1 .. " hq",
			["Explanation"] = "Opens the DarkruneDK HQ"
		},
	};
end

local function addonHelp()
	print("Following commands can be used with " .. DLU.addonNameColored .. ":");
	
	if (helpList == nil) then
		createHelpList();
	end
	
	for k,v in ipairs(helpList) do
		local command = DLU.colorizeString(v["Command"], "green");
		local text = v["Explanation"];
		print(format("%s (%s)", command, text));
	end
	
	print("You can use the commands with macros if you want easier/faster execution.");
end

local function CheckSettings()
	local charSettings = DLU.getDefaultCharacterSettings();
	
	for k,v in pairs(charSettings) do
		if (DLUCharacterSettings[k] == nil) then
			DLUCharacterSettings[k] = v;
		end
	end
	
	local settings = DLU.getDefaultOptions();
	
	for k,v in pairs(settings) do
		if (DLUSettings[k] == nil) then
			DLUSettings[k] = v;
		end
	end
end

-- Addon commands --
SLASH_RELOADUI1 = "/rl";
SlashCmdList.RELOADUI = ReloadUI;

SLASH_XP_HELP1 = "/dluhelp";
SlashCmdList.XP_HELP = function()
	print(format(addonData.L.HelpErrorText, addonName));
end

SLASH_DLU1 = "/dlu";
SlashCmdList.DLU = function(commandName)
	local command, arg2 = strsplit(" ", commandName);
	local arg = command:lower();

	if (arg == "") then
		InterfaceOptionsFrame_OpenToCategory(DLU.addonName);
	elseif (arg == "help") then
		addonHelp();
	elseif (arg == "test") then
		testIt();
	elseif (arg == "talentsave") then
		DLU.SaveTemplate(arg2);
	elseif (arg == "talentload") then
		DLU.LoadTemplate(arg2);
	elseif (arg == "talentdelete") then
		DLU.DeleteTemplate(arg2, true);
	elseif (arg == "il" or commandName:lower() == "ilvl") then
		local iLvl = DLU.getItemLevel();
		local _, _, _, _, _, _, _, _, _, _, _, _, superiorCompleted = GetAchievementInfo(12546);
		local _, _, _, _, _, _, _, _, _, _, _, _, epicCompleted = GetAchievementInfo(12547);
		if (epicCompleted and iLvl > 340) then
			iLvl = DLU.itemColorString(4, iLvl);
		elseif (superiorCompleted and iLvl > 310) then
			iLvl = DLU.itemColorString(3, iLvl);
		end

		print(string.format("Your item level is: %s", iLvl));
	elseif (arg == "ms") then
		local data = DLU.getMovementSpeedInfo();
		print(format("Ground speed: %i%% | Flying speed: %i%% | Swimming speed: %i%%", data.ground, data.flying, data.swimming));
	elseif (arg == "hq") then
		DLUHQ:Show();
	elseif (arg == "cm") then
		DLU.CheckClassMountStatus();
	else
		DLU.createMessage("Unknown command! Check '/dlu help' if you are uncertain.", "error");
	end
end

-- Registering Event --
DLUCoreFrame:RegisterEvent("PLAYER_LOGIN");
DLUCoreFrame:SetScript("OnEvent", function()
	DLU.getSettings();
	
	CheckSettings();
	
	-- Excecute --
	if (IsAddOnLoaded(DLU.addonName) and DLUSettings.showLoadMessage.value) then
		DLU.createMessage(format(addonData.L.Initialized .. " " .. addonData.L.HelpText, DLU.addonNameColored, SLASH_DLU1 .. " help"));
	end

	local d = C_DateAndTime.GetCurrentCalendarTime();
	if (d.month == 4 and d.monthDay == 1) then
		DLU.aprilsFool = true;
	end

	DLUCoreFrame:UnregisterEvent("PLAYER_LOGIN")
end);